Package: NLP
Version: 0.3-2
Title: Natural Language Processing Infrastructure
Authors@R: person("Kurt", "Hornik", role = c("aut", "cre"),
                  email = "Kurt.Hornik@R-project.org",
		  comment = c(ORCID = "0000-0003-4198-9911"))
Description: Basic classes and methods for Natural Language Processing.
License: GPL-3
Imports: utils
Depends: R (>= 3.5.0)
Enhances: udpipe, spacyr, cleanNLP
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2024-11-20 07:53:13 UTC; hornik
Author: Kurt Hornik [aut, cre] (<https://orcid.org/0000-0003-4198-9911>)
Maintainer: Kurt Hornik <Kurt.Hornik@R-project.org>
Repository: CRAN
Date/Publication: 2024-11-20 08:04:45 UTC
Built: R 4.6.0; ; 2026-04-25 16:16:06 UTC; unix
