Package: NLP
Version: 0.3-3
Title: Natural Language Processing Infrastructure
Authors@R: person("Kurt", "Hornik", role = c("aut", "cre"),
                  email = "Kurt.Hornik@R-project.org",
		  comment = c(ORCID = "0000-0003-4198-9911"))
Description: Basic classes and methods for Natural Language Processing.
License: GPL-3
Imports: utils
Depends: R (>= 3.5.0)
Enhances: udpipe, spacyr, cleanNLP
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-07-13 09:23:30 UTC; hornik
Author: Kurt Hornik [aut, cre] (ORCID: <https://orcid.org/0000-0003-4198-9911>)
Maintainer: Kurt Hornik <Kurt.Hornik@R-project.org>
Repository: CRAN
Date/Publication: 2026-07-13 09:45:34 UTC
